import bpy
from bpy.app.handlers import persistent

# Import From Files
if "bpy" in locals():
    import importlib
    from .. import helpers
    importlib.reload(helpers)
    from ..helpers import addon_prefs
    from . import colors
    importlib.reload(colors)
    from . import keymap
    importlib.reload(keymap)
else:
    from ..helpers import addon_prefs
    from . import colors
    from . import keymap

ADDON_NAME = "face2FACE"

current_projection_name = "NOTHING"


class SettingProps(colors.ColorProps):
    # global
    is_active: bpy.props.BoolProperty(
        name="is active",
        description="a flag that shows if the addon is active and all props are inited",
        default=False
    )

    is_modal_active: bpy.props.BoolProperty(
        name="is modal active",
        description="a flag that shows if the modal is active and used at the moment",
        default=False
    )

    # functional
    linked_copy: bpy.props.BoolProperty(
        name="Linked objects",
        description="determs wether the projected objects are linked or seperate ones",
        default=True
    )
    linked_from_source: bpy.props.BoolProperty(
        name="Link from source object",
        description="determs if the projected objects are linked from the source object",
        default=False
    )
    copy_when_single_target: bpy.props.BoolProperty(
        name="copy when single target",
        description="copy source object when there is only a single target face",
        default=False
    )
    delete_objects_also: bpy.props.BoolProperty(
        name="delete proj. objects also",
        description="determs wether the projected objects should be deleted with the projection",
        default=True
    )
    delete_target_object_also: bpy.props.BoolProperty(
        name="delete target object also",
        description="determs wether the target object should be deleted with the projection",
        default=False
    )
    max_scale_variation_strength: bpy.props.FloatProperty(
        name="max scale variation strength",
        description="sets the max scale variation strength of the projected objects",
        default=1.0,
        min=1.0,
        max=10.0,
        step=0.05
    )
    max_scale_relative_strength: bpy.props.FloatProperty(
        name="max scale relative strength",
        description="sets the max scale relative strength of the projected objects",
        default=1.0,
        min=1.0,
        max=10.0,
        step=0.05
    )
    shift_rotation_step: bpy.props.FloatProperty(
        name="shift rotation step",
        description="sets rotation step when shift + the step button is pressed",
        default=5.0,
        min=1.0,
        max=360.0,
        step=1
    )
    ctrl_rotation_step: bpy.props.FloatProperty(
        name="ctrl rotation step",
        description="sets rotation step when ctrl + the step button is pressed",
        default=15.0,
        min=1.0,
        max=360.0,
        step=1
    )
    normal_rotation_step: bpy.props.FloatProperty(
        name="normal rotation step",
        description="sets rotation step when nothing except the step button is pressed",
        default=30.0,
        min=1.0,
        max=360.0,
        step=1
    )
    alt_rotation_step: bpy.props.FloatProperty(
        name="alt rotation step",
        description="sets rotation step when alt + the step button is pressed",
        default=45.0,
        min=1.0,
        max=360.0,
        step=1
    )
    vary_rotation_step: bpy.props.FloatProperty(
        name="vary rotation step",
        description="sets the random roation variation step (0 means no step at all)",
        default=0.0,
        min=1.0,
        max=360.0,
        step=1
    )

    def set_vary_scale_step(self, value):
        #self.vary_scale_step["max"] = self.max_scale_relative_strength
        # if 0 <= value < 100:
        #self['vary_scale_step'] = 0.5
        self["value"] = 0.5
        return None
    vary_scale_step: bpy.props.FloatProperty(
        name="vary scale step",
        description="sets the random scale variation step (0 means no step at all)",
        default=0.0,
        min=0.0,
        max=10.0,  # self.max_scale_variation_strength,
        # step=100#,
        # set=set_vary_scale_step
    )

    # usability
    start_with_relative_pos: bpy.props.BoolProperty(
        name="start with relative pos",
        description="start with relative or absolute positioning",
        default=False
    )
    start_with_limit: bpy.props.BoolProperty(
        name="start with limit",
        description="start with limit on or off",
        default=False
    )

    # visual
    auto_view: bpy.props.BoolProperty(
        name="Auto align view",
        description="Automaticaly align view",
        default=False
    )
    highlight_projected_objects: bpy.props.BoolProperty(
        name="highlight projected objectes",
        description="visualy highlight our objects",
        default=True
    )

    highlight_rotation_flipped: bpy.props.BoolProperty(
        name="highlight rotation_ lipped",
        description="visualy highlight objects with flipped rotation",
        default=True
    )

    show_logo: bpy.props.BoolProperty(
        name="show logo",
        description="show logo insisde ui",
        default=True
    )

    # interface
    right_click_select: bpy.props.BoolProperty(
        name="right click select",
        description="right or left click select?",
        default=True
    )

    def draw(self, context, layout, header_text):
        col = layout.column()
        col.label(text=header_text)
        self.draw_functional(layout)
        self.draw_usability(layout)
        self.draw_visual(context, layout)

    def draw_functional(self, layout):

        if not self.linked_copy:
            self.linked_from_source = False
        if self.linked_copy and self.copy_when_single_target:
            self.linked_from_source = True

        if self.vary_scale_step > self.max_scale_variation_strength:
            self.vary_scale_step = self.max_scale_variation_strength

        # functionial
        box_outer = layout.box()
        col = box_outer.column()

        col.label(text="Creation:")
        box_inner = box_outer.box()
        col = box_inner.column(align=False)
        col.prop(self, 'linked_copy', expand=True)
        # if context.window_manager.face2face.linked_copy:
        if self.linked_copy:
            col.prop(self, 'linked_from_source')
        col.prop(self, 'copy_when_single_target', expand=True)

        col = box_outer.column()
        col.label(text="Delete:")
        box_inner = box_outer.box()
        col = box_inner.column(align=False)
        col.prop(self, 'delete_objects_also', expand=True)
        col.prop(self, 'delete_target_object_also', expand=True)

        col = box_outer.column()
        col.label(text="Max scale variation:")
        box_inner = box_outer.box()
        col = box_inner.column(align=False)
        col.prop(self, 'max_scale_variation_strength', slider=True)
        col.prop(self, 'max_scale_relative_strength', slider=True)

        col = box_outer.column()
        col.label(text="Step size for variation:")
        box_inner = box_outer.box()
        col = box_inner.column(align=False)
        col.prop(self, 'vary_rotation_step', slider=True)
        col.prop(self, 'vary_scale_step', slider=True)

    def draw_usability(self, layout):
        box_outer = layout.box()
        # useability
        col = box_outer.column()
        col.label(text="start options:")
        box_inner = box_outer.box()
        col = box_inner.column()
        col.prop(self, 'start_with_relative_pos', expand=True)
        col.prop(self, 'start_with_limit', expand=True)

        col = box_outer.column()
        col.label(text="Step rotation angles:")
        box_inner = box_outer.box()
        col = box_inner.column(align=False)
        col.prop(self, 'shift_rotation_step', slider=True)
        col.prop(self, 'ctrl_rotation_step', slider=True)
        col.prop(self, 'normal_rotation_step', slider=True)
        col.prop(self, 'alt_rotation_step', slider=True)

    def draw_visual(self, context, layout):
        box_outer = layout.box()
        # visual
        col = box_outer.column()
        col.label(text="Visual:")
        box_inner = box_outer.box()
        col = box_inner.column()
        col.prop(self, 'auto_view')
        col.prop(self, 'highlight_projected_objects')
        col.prop(self, 'highlight_rotation_flipped')
        col.prop(self, 'show_logo')

        self.draw_colors(context, layout, "Color theme")


class DrawMsg():
    # global
    blendout: bpy.props.BoolProperty(
        name="modal is blending out",
        description="a flag that shows if the addon is blending_out",
        default=False
    )
    do_cleanup: bpy.props.BoolProperty(
        name="do draw pre cleanup",
        description="a flag that shows if draw  callback should do a pre cleanup first",
        default=False
    )


class F2FSettings(DrawMsg, SettingProps, bpy.types.PropertyGroup):

    # right_click_select = True
    def is_select(self, event_type):
        if self.right_click_select:
            if event_type == 'RIGHTMOUSE':
                return True
        if not self.right_click_select:
            if event_type == 'LEFTMOUSE':
                return True
        return False

    def is_non_select(self, event_type):
        if not self.right_click_select:
            if event_type == 'RIGHTMOUSE':
                return True
        if self.right_click_select:
            if event_type == 'LEFTMOUSE':
                return True
        return False


@persistent
def save_props_per_file(scene):
    """ handle saving of files: store current session settings in scene so they could be saved """
    bpy.context.scene.face2face.is_active = bpy.context.window_manager.face2face.is_active
    bpy.context.scene.face2face.linked_copy = bpy.context.window_manager.face2face.linked_copy
    bpy.context.scene.face2face.linked_from_source = bpy.context.window_manager.face2face.linked_from_source
    bpy.context.scene.face2face.copy_when_single_target = bpy.context.window_manager.face2face.copy_when_single_target
    bpy.context.scene.face2face.delete_objects_also = bpy.context.window_manager.face2face.delete_objects_also
    bpy.context.scene.face2face.delete_target_object_also = bpy.context.window_manager.face2face.delete_target_object_also
    bpy.context.scene.face2face.max_scale_variation_strength = bpy.context.window_manager.face2face.max_scale_variation_strength
    bpy.context.scene.face2face.max_scale_relative_strength = bpy.context.window_manager.face2face.max_scale_relative_strength
    bpy.context.scene.face2face.shift_rotation_step = bpy.context.window_manager.face2face.shift_rotation_step
    bpy.context.scene.face2face.ctrl_rotation_step = bpy.context.window_manager.face2face.ctrl_rotation_step
    bpy.context.scene.face2face.normal_rotation_step = bpy.context.window_manager.face2face.normal_rotation_step
    bpy.context.scene.face2face.alt_rotation_step = bpy.context.window_manager.face2face.alt_rotation_step
    bpy.context.scene.face2face.vary_rotation_step = bpy.context.window_manager.face2face.vary_rotation_step
    bpy.context.scene.face2face.vary_scale_step = bpy.context.window_manager.face2face.vary_scale_step
    bpy.context.scene.face2face.start_with_relative_pos = bpy.context.window_manager.face2face.start_with_relative_pos
    bpy.context.scene.face2face.start_with_limit = bpy.context.window_manager.face2face.start_with_limit
    bpy.context.scene.face2face.auto_view = bpy.context.window_manager.face2face.auto_view
    bpy.context.scene.face2face.highlight_projected_objects = bpy.context.window_manager.face2face.highlight_projected_objects
    bpy.context.scene.face2face.show_logo = bpy.context.window_manager.face2face.show_logo
    bpy.context.scene.face2face.status_text_color = bpy.context.window_manager.face2face.status_text_color
    bpy.context.scene.face2face.status_text_header_color = bpy.context.window_manager.face2face.status_text_header_color
    bpy.context.scene.face2face.status_text_error_color = bpy.context.window_manager.face2face.status_text_error_color
    bpy.context.scene.face2face.highlight_projected_objects_color = bpy.context.window_manager.face2face.highlight_projected_objects_color
    bpy.context.scene.face2face.highlight_rotation_flipped_color = bpy.context.window_manager.face2face.highlight_rotation_flipped_color


@ persistent
def load_props_per_file(scene):
    """ handle loading of files: use loaded settings, activate if needed or init settings """
    if not bpy.context.window_manager.face2face.is_active:
        if bpy.context.scene.face2face.is_active:
            # loaded file has projections so activate and init props
            bpy.ops.object.activate_face2face()
        else:
            # load global defaults into local prefs
            bpy.context.window_manager.face2face.linked_copy = addon_prefs(
                bpy.context).linked_copy
            bpy.context.window_manager.face2face.linked_from_source = addon_prefs(
                bpy.context).linked_from_source
            bpy.context.window_manager.face2face.copy_when_single_target = addon_prefs(
                bpy.context).copy_when_single_target
            bpy.context.window_manager.face2face.delete_objects_also = addon_prefs(
                bpy.context).delete_objects_also
            bpy.context.window_manager.face2face.delete_target_object_also = addon_prefs(
                bpy.context).delete_target_object_also
            bpy.context.window_manager.face2face.max_scale_variation_strength = addon_prefs(
                bpy.context).max_scale_variation_strength
            bpy.context.window_manager.face2face.max_scale_relative_strength = addon_prefs(
                bpy.context).max_scale_relative_strength
            bpy.context.window_manager.face2face.normal_rotation_step = addon_prefs(
                bpy.context).normal_rotation_step
            bpy.context.window_manager.face2face.alt_rotation_step = addon_prefs(
                bpy.context).alt_rotation_step
            bpy.context.window_manager.face2face.vary_rotation_step = addon_prefs(
                bpy.context).vary_rotation_step
            bpy.context.window_manager.face2face.vary_scale_step = addon_prefs(
                bpy.context).vary_scale_step
            bpy.context.window_manager.face2face.ctrl_rotation_step = addon_prefs(
                bpy.context).ctrl_rotation_step
            bpy.context.window_manager.face2face.shift_rotation_step = addon_prefs(
                bpy.context).shift_rotation_step
            bpy.context.window_manager.face2face.start_with_relative_pos = addon_prefs(
                bpy.context).start_with_relative_pos
            bpy.context.window_manager.face2face.start_with_limit = addon_prefs(
                bpy.context).start_with_limit
            bpy.context.window_manager.face2face.auto_view = addon_prefs(
                bpy.context).auto_view
            bpy.context.window_manager.face2face.highlight_projected_objects = addon_prefs(
                bpy.context).highlight_projected_objects
            bpy.context.window_manager.face2face.show_logo = addon_prefs(
                bpy.context).show_logo
            bpy.context.window_manager.face2face.status_text_color = addon_prefs(
                bpy.context).status_text_color
            bpy.context.window_manager.face2face.status_text_header_color = addon_prefs(
                bpy.context).status_text_header_color
            bpy.context.window_manager.face2face.status_text_error_color = addon_prefs(
                bpy.context).status_text_error_color
            bpy.context.window_manager.face2face.highlight_projected_objects_color = addon_prefs(
                bpy.context).highlight_projected_objects_color
            bpy.context.window_manager.face2face.highlight_rotation_flipped_color = addon_prefs(
                bpy.context).highlight_rotation_flipped_color

    if bpy.context.scene.face2face.is_active:
        # loaded file has projections ... so use file settings
        bpy.context.window_manager.face2face.is_active = bpy.context.scene.face2face.is_active
        bpy.context.window_manager.face2face.linked_copy = bpy.context.scene.face2face.linked_copy
        bpy.context.window_manager.face2face.linked_from_source = bpy.context.scene.face2face.linked_from_source
        bpy.context.window_manager.face2face.copy_when_single_target = bpy.context.scene.face2face.copy_when_single_target
        bpy.context.window_manager.face2face.delete_objects_also = bpy.context.scene.face2face.delete_objects_also
        bpy.context.window_manager.face2face.delete_target_object_also = bpy.context.scene.face2face.delete_target_object_also
        bpy.context.window_manager.face2face.max_scale_variation_strength = bpy.context.scene.face2face.max_scale_variation_strength
        bpy.context.window_manager.face2face.max_scale_relative_strength = bpy.context.scene.face2face.max_scale_relative_strength
        bpy.context.window_manager.face2face.shift_rotation_step = bpy.context.scene.face2face.shift_rotation_step
        bpy.context.window_manager.face2face.ctrl_rotation_step = bpy.context.scene.face2face.ctrl_rotation_step
        bpy.context.window_manager.face2face.normal_rotation_step = bpy.context.scene.face2face.normal_rotation_step
        bpy.context.window_manager.face2face.alt_rotation_step = bpy.context.scene.face2face.alt_rotation_step
        bpy.context.window_manager.face2face.vary_rotation_step = bpy.context.scene.face2face.vary_rotation_step
        bpy.context.window_manager.face2face.vary_scale_step = bpy.context.scene.face2face.vary_scale_step
        bpy.context.window_manager.face2face.start_with_limit = bpy.context.scene.face2face.start_with_relative_pos
        bpy.context.window_manager.face2face.start_with_limit = bpy.context.scene.face2face.start_with_limit
        bpy.context.window_manager.face2face.auto_view = bpy.context.scene.face2face.auto_view
        bpy.context.window_manager.face2face.highlight_projected_objects = bpy.context.scene.face2face.highlight_projected_objects
        bpy.context.window_manager.face2face.show_logo = bpy.context.scene.face2face.show_logo
        bpy.context.window_manager.face2face.status_text_color = bpy.context.scene.face2face.status_text_color
        bpy.context.window_manager.face2face.status_text_header_color = bpy.context.scene.face2face.status_text_header_color
        bpy.context.window_manager.face2face.status_text_error_color = bpy.context.scene.face2face.status_text_error_color
        bpy.context.window_manager.face2face.highlight_projected_objects_color = bpy.context.scene.face2face.highlight_projected_objects_color
        bpy.context.window_manager.face2face.highlight_rotation_flipped_color = bpy.context.scene.face2face.highlight_rotation_flipped_color


class f2f_tmp_vars:
    objects_projected = 0
    options_panel_msg = None
    circle_selecting = []
    box_selecting = []
    update_rotflip_objects = False
    rotflip_objects = []
    selection_changed = True


def register():
    """ register all operator classes """
    bpy.utils.register_class(F2FSettings)
    f2f = bpy.props.PointerProperty(type=F2FSettings)
    bpy.types.Scene.face2face = f2f
    bpy.types.WindowManager.face2face = f2f
    bpy.types.WindowManager.f2f_tmp_vars = f2f_tmp_vars()
    if not save_props_per_file in bpy.app.handlers.save_pre:
        bpy.app.handlers.save_pre.append(save_props_per_file)
    if not load_props_per_file in bpy.app.handlers.load_post:
        bpy.app.handlers.load_post.append(load_props_per_file)

    # addon reloaded after activation needs to set props again
    if bpy.context.window_manager.face2face.is_active:
        bpy.context.window_manager.face2face.is_active = False
        # print("ADDON REELOAD activated!!!!!!!!!!!!!!!!!!!!!!!!!")


def unregister():
    """ unregister all operator classes """
    bpy.utils.unregister_class(F2FSettings)

    if save_props_per_file in bpy.app.handlers.save_pre:
        bpy.app.handlers.save_pre.remove(save_props_per_file)
    if load_props_per_file in bpy.app.handlers.load_post:
        bpy.app.handlers.load_post.remove(load_props_per_file)

    del bpy.types.WindowManager.face2face
    del bpy.types.Scene.face2face
