""" This file contains function to set and restore the status bar layout"""
import bpy
from bl_ui import space_statusbar

# Import From Files
if "bpy" in locals():
    import importlib
    from .. import states
    importlib.reload(states)
    from ..config import keymap
    importlib.reload(keymap)
else:
    from .. import states
    from ..config import keymap

statusbar_header = space_statusbar.STATUSBAR_HT_header
original_draw_handler = None
state = None #states.IdleState()


def init():
    global original_draw_handler

    if not original_draw_handler:
        original_draw_handler = statusbar_header.draw

    statusbar_header.draw = draw_handler


def restore():
    statusbar_header.draw = original_draw_handler


def set_layout_by_state(layout):
    """ adjust layout to modal state """
    global state
    if isinstance(state, states.ScaleState):
        set_scaling_layout(layout)
    if isinstance(state, states.IdleState):
        set_idle_layout(layout)
    """ if self.next_state == 'BLENDOUT':
        self.state = BlendOutState()
        self.current_state = 'BLENDOUT' """
    if isinstance(state, states.GrabState):
        set_grabing_layout(layout)
    if isinstance(state, states.RotateState):
        set_rotating_layout(layout)


def set_idle_layout(layout):
    sep = ''
    row = layout.row(align=True)
    row.label(text=F'{sep}CONFIRM FF', icon='MOUSE_LMB')
    layout.separator(factor=0.5)
    row = layout.row(align=True)
    row.label(text=F'{sep}CANCEL All', icon='MOUSE_RMB')

    if keymap.view_on_middlemouse:
        layout.separator(factor=3.0)
        row = layout.row(align=True)  
        row.label(text=F'{sep}VIEW', icon='MOUSE_MMB')

    layout.separator(factor=3.0)
    row = layout.row(align=True)
    row.label(text=F'{sep}MOVE', icon='EVENT_G')
    layout.separator(factor=1.0)
    row = layout.row(align=True)
    row.label(text=F'{sep}SCALE', icon='EVENT_S')
    layout.separator(factor=1.0)
    row = layout.row(align=True)
    row.label(text=F'{sep}ROTATE', icon='EVENT_R')

    layout.separator(factor=3.0)
    row = layout.row(align=True)
    row.label(text=F'{sep} ADAPT TRANSFORMATION', icon='EVENT_A')
    layout.separator(factor=1.0)
    row = layout.row(align=True)
    row.label(
        text=F'{sep} ORIGINAL TRANSFORMATION', icon='EVENT_O')


def set_scaling_layout(layout):
    if state.setting_variation:
        set_scaling_vary_layout(layout)
    elif state.setting_relative:
        set_scaling_relative_layout(layout)
    else:
        set_scaling_normal_layout(layout)


def set_scaling_vary_layout(layout):
    global state
    sep = ''
    row = layout.row(align=True)
    row.label(text=F'{sep}ADJUST', icon='MOUSE_MOVE')
    layout.separator(factor=0.5)
    row = layout.row(align=True)
    row.label(text=F'{sep}CONFIRM variation strength', icon='MOUSE_LMB')
    layout.separator(factor=0.5)
    row = layout.row(align=True)
    row.label(text=F'{sep}CANCEL variation strength', icon='MOUSE_RMB')

    if keymap.view_on_middlemouse:
        layout.separator(factor=3.0)
        row = layout.row(align=True)  
        row.label(text=F'{sep}VIEW', icon='MOUSE_MMB')

    layout.separator(factor=3.0)
    row = layout.row(align=True)
    vary_text = "RESHUFFLE variation per object"
    row.label(text=F'{sep} '+vary_text, icon='EVENT_V')

    layout.separator(factor=1.0)
    row = layout.row(align=True)
    vary_text = "set variation per object to zero"
    row.label(text=F'{sep} '+vary_text, icon='EVENT_Z')


def set_scaling_relative_layout(layout):
    global state
    sep = ''
    row = layout.row(align=True)
    row.label(text=F'{sep}ADJUST', icon='MOUSE_MOVE')
    layout.separator(factor=0.5)
    row = layout.row(align=True)
    row.label(text=F'{sep}CONFIRM face relation strength', icon='MOUSE_LMB')
    layout.separator(factor=0.5)
    row = layout.row(align=True)
    row.label(text=F'{sep}CANCEL face relation strength', icon='MOUSE_RMB')

    if keymap.view_on_middlemouse:
        layout.separator(factor=3.0)
        row = layout.row(align=True)  
        row.label(text=F'{sep}VIEW', icon='MOUSE_MMB')


def set_scaling_normal_layout(layout):
    global state
    sep = ''
    row = layout.row(align=True)
    if not state.fit_scale:
        row.label(text=F'{sep}ADJUST', icon='MOUSE_MOVE')
        layout.separator(factor=0.5)
        row = layout.row(align=True)
    row.label(text=F'{sep}CONFIRM scaling', icon='MOUSE_LMB')
    layout.separator(factor=0.5)
    row = layout.row(align=True)
    row.label(text=F'{sep}CANCEL scaling', icon='MOUSE_RMB')

    if keymap.view_on_middlemouse:
        layout.separator(factor=3.0)
        row = layout.row(align=True)  
        row.label(text=F'{sep}VIEW', icon='MOUSE_MMB')

    """ layout.separator(factor=1.0)
    row = layout.row(align=True)
    fit_scale_text = "turn STRETCH FIT ON"
    if state.fit_scale:
        fit_scale_text = "turn STRETCH FIT OFF"
    row.label(text=F'{sep} '+fit_scale_text, icon='EVENT_S') """
    layout.separator(factor=3.0)
    row = layout.row(align=True)
    if state.fit_scale:
        fit_scale_text = "switch 2 EVEN"
        if state.even_scale:
            fit_scale_text = "switch 2 INDIVIDUAL MAX"
            row.label(text=F'{sep} '+fit_scale_text, icon='EVENT_I')
            layout.separator(factor=1.0)
            row = layout.row(align=True)
            fit_scale_text = "turn STRETCH FIT OFF"
            row.label(text='', icon='EVENT_E')
            row.label(text=F'{sep} '+fit_scale_text, icon='EVENT_S')
        else:
            row.label(text=F'{sep} '+fit_scale_text, icon='EVENT_E')
            layout.separator(factor=1.0)
            row = layout.row(align=True)
            fit_scale_text = "turn STRETCH FIT OFF"
            row.label(text='', icon='EVENT_I')
            row.label(text=F'{sep} '+fit_scale_text, icon='EVENT_S')
    else:
        fit_scale_text = "turn STRETCH FIT EVEN ON"
        row.label(text=F'{sep} '+fit_scale_text, icon='EVENT_E')
        layout.separator(factor=1.0)
        row = layout.row(align=True)
        fit_scale_text = "turn STRETCH FIT I-MAX ON"
        row.label(text=F'{sep} '+fit_scale_text, icon='EVENT_I')

    if not state.fit_scale:
        layout.separator(factor=1.0)
        row = layout.row(align=True)
        vary_text = "set VARIATION"
        row.label(text=F'{sep} '+vary_text, icon='EVENT_V')

        layout.separator(factor=1.0)
        row = layout.row(align=True)
        vary_text = "set RELATION"
        row.label(text=F'{sep} '+vary_text, icon='EVENT_R')

    #if not state.fit_scale:
        layout.separator(factor=1.0)
        row = layout.row(align=True)
        limit_text = "turn LIMIT ON"
        if state.limit_to_face:
            limit_text = "turn LIMIT OFF"
        row.label(text=F'{sep} '+limit_text, icon='EVENT_L')


    layout.separator(factor=2.0)
    row = layout.row(align=True)
    row.label(text='', icon='EVENT_X')
    if state.fit_scale:
        ltext = F'{sep} lock to axis'
    else:
        ltext = ''
    row.label(text=ltext, icon='EVENT_Y')
    if not state.fit_scale:
        row.label(text=F'{sep} lock to axis', icon='EVENT_Z')

    layout.separator(factor=2.0)
    row = layout.row(align=True)
    row.label(text='', icon='EVENT_ALT')
    row.label(text=F'{sep} 1-9 PRECISE INPUT')

    layout.separator(factor=2.0)
    row = layout.row(align=True)
    row.label(text=F'{sep} ADAPT SCALE', icon='EVENT_A')
    layout.separator(factor=1.0)
    row = layout.row(align=True)
    row.label(text=F'{sep} ORIGINAL SCALE', icon='EVENT_O')

    

def set_grabing_layout(layout):
    global state
    sep = ''
    row = layout.row(align=True)
    row.label(text=F'{sep}ADJUST', icon='MOUSE_MOVE')
    layout.separator(factor=0.5)
    row = layout.row(align=True)
    row.label(text=F'{sep}CONFIRM move', icon='MOUSE_LMB')
    layout.separator(factor=0.5)
    row = layout.row(align=True)
    row.label(text=F'{sep}CANCEL move', icon='MOUSE_RMB')

    if keymap.view_on_middlemouse:
        layout.separator(factor=3.0)
        row = layout.row(align=True)  
        row.label(text=F'{sep}VIEW', icon='MOUSE_MMB')

    layout.separator(factor=3.0)
    row = layout.row(align=True)
    positioning_text = "RELATIVE mode"
    if state.relative_pos:
        positioning_text = "ABSOLUTE mode"
    #positioning_text = "position current only/absolute/relative"
    #positioning_text = "positioning MODE"
    if state.relative_pos: 
        row.label(text='', icon='EVENT_A')
    else:
        row.label(text='', icon='EVENT_R')
    row.label(text=F'{sep} '+positioning_text)

    layout.separator(factor=1.0)
    row = layout.row(align=True)
    if not state.relative_pos:
        limit_text = "turn LIMIT ON"
        if state.limit_to_face:
            limit_text = "turn LIMIT OFF"
        row.label(text=F'{sep} '+limit_text, icon='EVENT_L')

        layout.separator(factor=1.0)
        row = layout.row(align=True)

        #for num in state.target_face_types:
        #    row.label(text=F'{sep}', icon='EVENT_F'+str(num))
        # row.label(text=F'{sep} ', icon='EVENT_F4')
        # row.label(text=F'{sep} select by verts', icon='EVENT_F4')
        #row.label(text=F'{sep} select by verts')
        #layout.separator(factor=1.0)
        #row = layout.row(align=True)

    row.label(text='', icon='EVENT_X')
    row.label(text='', icon='EVENT_Y')
    row.label(text=F'{sep} lock to axis', icon='EVENT_Z')

    layout.separator(factor=2.0)
    row = layout.row(align=True)
    row.label(text='', icon='EVENT_ALT')
    row.label(text=F'{sep} 1-9 PRECISE INPUT')

    layout.separator(factor=2.0)
    row = layout.row(align=True)
    if not state.relative_pos:
        row.label(text=F'{sep} ADAPT POSITION', icon='EVENT_A')
        layout.separator(factor=1.0)
        row = layout.row(align=True)
    row.label(text=F'{sep} RECENTER', icon='EVENT_O')

def set_selction_keys_layout(layout, context):
    global state
    sep = ''
    layout.separator(factor=1.0)

    box = layout.box()
    box.label(text=F'{sep} Selection Modes:')
    #layout.separator(factor=1.0)
    row = layout.row(align=True)
    row.label(text='', icon='TRIA_UP')
    row.label(text=F'{sep} PREV/NEXT', icon='TRIA_DOWN')

    #layout.separator(factor=1.0)
    row = layout.row(align=True)
    row.label(text=F'{sep} 1 All')
    row = layout.row(align=True)
    #row.label(text=F'{sep} 3 ,4-9 Tris,Quads,N-gon')
    row.label(text=F'{sep} 3-9 All Tris,Quads or N-gons')


    #layout.separator(factor=1.0)
    #row = layout.row(align=True)
    #row.label(text='Manual(Normal) selection', icon='EVENT_M')

    row = layout.row(align=True)
    row.label(text='', icon='EVENT_ALT')
    if not context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}Select object', icon='MOUSE_LMB')
    if context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}Select object', icon='MOUSE_RMB')

    row = layout.row(align=True)
    row.label(text='', icon='EVENT_SHIFT')
    if not context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}toggle Selection',
                  icon='MOUSE_LMB')
    if context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}toggle Selection',
                  icon='MOUSE_RMB')


    row = layout.row(align=True)
    row.label(text='Type selection', icon='EVENT_T')
    row = layout.row(align=True)
    row.label(text='Faceloop selection', icon='EVENT_F')
    row = layout.row(align=True)
    row.label(text='Circle selection', icon='EVENT_C')
    row = layout.row(align=True)
    row.label(text='Box selection', icon='EVENT_B')
    

def set_rotating_layout(layout):
    if state.setting_variation:
        set_rotating_vary_layout(layout)
    else:
        set_rotating_normal_layout(layout)


def set_rotating_normal_layout(layout):
    sep = ''
    row = layout.row(align=True)
    row.label(text=F'{sep}ADJUST', icon='MOUSE_MOVE')
    layout.separator(factor=0.5)
    row = layout.row(align=True)
    row.label(text=F'{sep}CONFIRM rotation', icon='MOUSE_LMB')
    layout.separator(factor=0.5)
    row = layout.row(align=True)
    row.label(text=F'{sep}CANCEL rotation', icon='MOUSE_RMB')

    if keymap.view_on_middlemouse:
        layout.separator(factor=3.0)
        row = layout.row(align=True)  
        row.label(text=F'{sep}VIEW', icon='MOUSE_MMB')

    """ layout.separator(factor=3.0)
    row = layout.row(align=True)
    row.label(text=F'{sep} set fix angle', icon='EVENT_A') """

    layout.separator(factor=3.0)
    row = layout.row(align=True)
    vary_text = "set VARIATION"
    row.label(text=F'{sep} '+vary_text, icon='EVENT_V')

    layout.separator(factor=2.0)
    row = layout.row(align=True)
    row.label(text=F'{sep} FLIP ROTATION', icon='EVENT_I')

    layout.separator(factor=2.0)
    row = layout.row(align=True)
    row.label(text=F'{sep} align to source edge', icon='EVENT_E')
    row = layout.row(align=True)
    row.label(text='', icon='EVENT_ALT')
    
    row.label(text=F'{sep} align to target edge', icon='EVENT_E')
    row = layout.row(align=True)
    #row.label(text=F'{sep}+ ')
    row.label(icon='EVENT_SHIFT')
    row.label(text='', icon='PLUS')
    row.label(text=F'{sep}align keys: counterclockwise')

    layout.separator(factor=2.0)
    row = layout.row(align=True)
    row.separator(factor=0)
    row.label(text='', icon='EVENT_SHIFT')
    #row.label(text='', icon='THREE_DOTS')
    row.label(text='', icon='EVENT_CTRL')
    #row.label(text='', icon='THREE_DOTS')
    row.label(text='', icon='EVENT_ALT')
    row.label(text='', icon='PLUS')
    row.label(text='', icon='TRIA_LEFT')
    row.label(text=' STEP', icon='TRIA_RIGHT')
    #row.label(text=F'{sep} STEP', icon='TRIA_RIGHT')

    layout.separator(factor=2.0)
    row = layout.row(align=True)
    row.label(text='', icon='EVENT_ALT')
    row.label(text=F'{sep} 1-9 PRECISE INPUT')

    layout.separator(factor=2.0)
    row = layout.row(align=True)
    row.label(text=F'{sep} ADAPT ROTATION', icon='EVENT_A')
    layout.separator(factor=1.0)
    row = layout.row(align=True)
    row.label(text=F'{sep} ORIGINAL ROTATION', icon='EVENT_O')


def set_rotating_vary_layout(layout):
    global state
    sep = ''
    row = layout.row(align=True)
    row.label(text=F'{sep}ADJUST', icon='MOUSE_MOVE')
    layout.separator(factor=0.5)
    row = layout.row(align=True)
    row.label(text=F'{sep}CONFIRM variation strength', icon='MOUSE_LMB')
    layout.separator(factor=0.5)
    row = layout.row(align=True)
    row.label(text=F'{sep}CANCEL variation strength', icon='MOUSE_RMB')

    if keymap.view_on_middlemouse:
        layout.separator(factor=3.0)
        row = layout.row(align=True)  
        row.label(text=F'{sep}VIEW', icon='MOUSE_MMB')

    layout.separator(factor=1.0)
    row = layout.row(align=True)
    vary_text = "RESHUFFLE variation per object"
    row.label(text=F'{sep} '+vary_text, icon='EVENT_V')

    layout.separator(factor=1.0)
    row = layout.row(align=True)
    vary_text = "set variation per object to zero"
    row.label(text=F'{sep} '+vary_text, icon='EVENT_Z')


def set_num_enter_layout(layout, context):
    global state
    sep = ''
    row = layout.row(align=True)
    
    row.label(text='', icon='EVENT_RETURN')
    row.label(text=F'{sep}Apply', icon='MOUSE_LMB')

    layout.separator(factor=5.0)
    row = layout.row(align=True)
    row.label(text=F'{sep} 1-9 for degree')

    layout.separator(factor=2.0)
    row = layout.row(align=True)
    row.label(text='', icon='DOT')
    row.label(text=F'{sep} Enter decimal places')

    layout.separator(factor=2.0)
    row = layout.row(align=True)
    row.label(text='', icon='BACK')
    row.label(text=F'{sep} Undo last')

    if isinstance(state, states.ScaleState) or isinstance(state, states.GrabState):
        layout.separator(factor=2.0)
        row = layout.row(align=True)
        row.label(text='', icon='EVENT_X')
        row.label(text='', icon='EVENT_Y')
        row.label(text=F'{sep} lock to axis', icon='EVENT_Z')

    layout.separator(factor=5.0)
    row = layout.row(align=True)
    row.label(text='', icon='EVENT_ESC')
    row.label(text=F'{sep}Cancel', icon='MOUSE_RMB')

def set_manual_selection_layout(layout, context):
    global state
    sep = ''
    row = layout.row(align=True)

    if not context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}Select object', icon='MOUSE_LMB')
    if context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}Select object', icon='MOUSE_RMB')

    layout.separator(factor=2.0)
    row = layout.row(align=True)
    row.label(text='', icon='EVENT_SHIFT')
    if not context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}toggle Selection',
                  icon='MOUSE_LMB')
    if context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}toggle Selection',
                  icon='MOUSE_RMB')

    layout.separator(factor=10.0)
    row = layout.row(align=True)
    row.label(text='', icon='EVENT_ESC')
    if context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}Cancel',
                  icon='MOUSE_LMB')
    if not context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}Cancel',
                  icon='MOUSE_RMB')


def set_object_type_selection_layout(layout, context):
    global state
    sep = ''
    row = layout.row(align=True)

    if not context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}Select object', icon='MOUSE_LMB')
    if context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}Select object', icon='MOUSE_RMB')

    layout.separator(factor=2.0)
    row = layout.row(align=True)
    row.label(text='', icon='EVENT_SHIFT')
    if not context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}add object types to Selection',
                  icon='MOUSE_LMB')
    if context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}add object to Selection',
                  icon='MOUSE_RMB')

    layout.separator(factor=2.0)
    row = layout.row(align=True)
    row.label(text='', icon='EVENT_CTRL')
    if not context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}remove object types to Selection',
                  icon='MOUSE_LMB')
    if context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}remove object to Selection',
                  icon='MOUSE_RMB')
    print("add/remove object to Selection")

    layout.separator(factor=10.0)
    row = layout.row(align=True)
    row.label(text='', icon='EVENT_ESC')
    if context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}Cancel',
                  icon='MOUSE_LMB')
    if not context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}Cancel',
                  icon='MOUSE_RMB')


def set_circle_selection_layout(layout, context):
    global state
    sep = ''
    row = layout.row(align=True)

    if context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}Select', icon='MOUSE_LMB')
    if not context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}Select', icon='MOUSE_RMB')

    layout.separator(factor=2.0)
    row = layout.row(align=True)
    row.label(text='', icon='EVENT_SHIFT')
    if context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}Deselect',
                  icon='MOUSE_LMB')
    if not context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}Deselect',
                  icon='MOUSE_RMB')

    layout.separator(factor=2.0)
    row = layout.row(align=True)
    row.label(text='WhUP:Add')
    layout.separator(factor=2.0)
    row = layout.row(align=True)
    row.label(text='WhDown:Subtract')

    layout.separator(factor=10.0)
    row = layout.row(align=True)
    row.label(text='', icon='EVENT_ESC')
    if not context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}LEAVE circle selection mode',
                  icon='MOUSE_LMB')
    if context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}LEAVE circle selection mode',
                  icon='MOUSE_RMB')


def set_box_selection_layout(layout, context):
    global state
    sep = ''
    row = layout.row(align=True)

    if context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}Select', icon='MOUSE_LMB')
    if not context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}Select', icon='MOUSE_RMB')

    layout.separator(factor=2.0)
    row = layout.row(align=True)
    row.label(text='', icon='EVENT_SHIFT')
    if context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}Deselect',
                  icon='MOUSE_LMB')
    if not context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}Deselect',
                  icon='MOUSE_RMB')

    layout.separator(factor=10.0)
    row = layout.row(align=True)
    row.label(text='', icon='EVENT_ESC')
    if not context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}LEAVE box selection mode',
                  icon='MOUSE_LMB')
    if context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}LEAVE box selection mode',
                  icon='MOUSE_RMB')


def set_face_loop_selection_layout(layout, context):
    global state
    sep = ''
    row = layout.row(align=True)

    if not context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}Select loop', icon='MOUSE_LMB')
    if context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}Select loop', icon='MOUSE_RMB')

    layout.separator(factor=2.0)
    row = layout.row(align=True)
    row.label(text='', icon='EVENT_ALT')
    if not context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}Select other loop', icon='MOUSE_LMB')
    if context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}Select other loop', icon='MOUSE_RMB')

    layout.separator(factor=2.0)
    row = layout.row(align=True)
    row.label(text='', icon='EVENT_SHIFT')
    if not context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}add loop to Selection',
                  icon='MOUSE_LMB')
    if context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}add loop to Selection',
                  icon='MOUSE_RMB')

    layout.separator(factor=2.0)
    row = layout.row(align=True)
    row.label(text='', icon='EVENT_ALT')
    row.label(text='', icon='EVENT_SHIFT')
    if not context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}add other loop to Selection',
                  icon='MOUSE_LMB')
    if context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}add other loop to Selection',
                  icon='MOUSE_RMB')

    layout.separator(factor=2.0)
    row = layout.row(align=True)
    row.label(text='', icon='EVENT_CTRL')
    if not context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}remove loop from Selection',
                  icon='MOUSE_LMB')
    if context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}remove loop from Selection',
                  icon='MOUSE_RMB')

    layout.separator(factor=2.0)
    row = layout.row(align=True)
    row.label(text='', icon='EVENT_ALT')
    row.label(text='', icon='EVENT_CTRL')
    if not context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}remove other loop from Selection',
                  icon='MOUSE_LMB')
    if context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}remove other loop from Selection',
                  icon='MOUSE_RMB')

    layout.separator(factor=10.0)
    row = layout.row(align=True)
    row.label(text='', icon='EVENT_ESC')
    if context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}Cancel',
                  icon='MOUSE_LMB')
    if not context.window_manager.face2face.right_click_select:
        row.label(text=F'{sep}Cancel',
                  icon='MOUSE_RMB')


def set_selection_layout(layout, context):
    global state
    if state.select_mode == 1:
        set_manual_selection_layout(layout, context)
    if state.select_mode == 2:
        set_object_type_selection_layout(layout, context)
    if state.select_mode == 3:
        set_circle_selection_layout(layout, context)
    if state.select_mode == 4:
        set_box_selection_layout(layout, context)
    if state.select_mode == 5:
        set_face_loop_selection_layout(layout, context)


def draw_handler(header, context):
    global state
    layout = header.layout
    draw2layout(layout, context)
    

def draw2layout(layout, context):
    if state:
        if state.select_mode > 0:
            set_selection_layout(layout, context)
        elif state.num_enter_mode:
            set_num_enter_layout(layout, context)
        else:
            set_layout_by_state(layout)

    if bpy.app.version < (2, 90, 0):
        layout.template_reports_banner()
        layout.template_running_jobs()

    #layout.separator_spacer()
