""" Functions for manipulating the view """
import bpy

# Import From Files
if "bpy" in locals():
    import importlib
    from . import mesh
    importlib.reload(mesh)
else:
    from . import mesh


def align_view2obj(context, obj, use_active):
    """ rotate view to a obj local transformation and
        center the view to object pivot or active face """
    # rotate view to objeects local orientation
    rotate_view2obj(context, obj)

    # and finally center view to objects pivot or active face
    if use_active:
        obj_location = mesh.get_midpoint_by_active(obj)
    else:
        obj_location = obj.matrix_world.to_translation()
    center_view2location(context, obj_location)


def align_view2custom_transform_target_face(context, obj, target_obj):
    """ rotate view to obj customs transformation of its target face
        and center the view to the middle of the face """
    rotate_view2custom_transform(context, obj.matrix_custom_transform)

    target_face = target_obj.data.polygons[obj.target_face_index]
    target_face_verts = mesh.get_verts_from_polygon(
        target_obj, target_face)
    target_face_mid = mesh.get_midpoint_world(target_obj, target_face_verts)
    center_view2location(context, target_face_mid)


def align_view2custom_transform_target_face_z(context, obj, target_obj):
    """ rotate view to obj customs transformation of its target face
        and center the view to the middle of the face """
    rotate_view2custom_transform_z(context, obj.matrix_custom_transform)

    target_face = target_obj.data.polygons[obj.target_face_index]
    target_face_verts = mesh.get_verts_from_polygon(
        target_obj, target_face)
    target_face_mid = mesh.get_midpoint_world(target_obj, target_face_verts)
    center_view2location(context, target_face_mid)


def align_view2custom_transform_target_face_ole(context, obj, target_obj):
    """ rotate view to obj customs transformation of its target face
        and center the view to the middle of the face """
    custom_transform = get_custom_transform_by_obj(context, obj)
    rotate_view2custom_transform(context, custom_transform)

    target_face = target_obj.data.polygons[obj.target_face_index]
    target_face_verts = mesh.get_verts_from_polygon(
        target_obj, target_face)
    target_face_mid = mesh.get_midpoint_world(target_obj, target_face_verts)
    center_view2location(context, target_face_mid)


def align_view2custom_transform_target_face_z_ole(context, obj, target_obj):
    """ rotate view to obj customs transformation of its target face
        and center the view to the middle of the face """
    custom_transform = get_custom_transform_by_obj(context, obj)
    rotate_view2custom_transform_z(context, custom_transform)

    target_face = target_obj.data.polygons[obj.target_face_index]
    target_face_verts = mesh.get_verts_from_polygon(
        target_obj, target_face)
    target_face_mid = mesh.get_midpoint_world(target_obj, target_face_verts)
    center_view2location(context, target_face_mid)


def get_custom_transform_by_obj(context, obj):
    """ find and return objects custom target face transformation by """
    ct_name = "Face2Align"+(str(obj.target_face_index))
    return get_custom_transform_by_name(context, ct_name)


def get_custom_transform_by_name(context, ct_name):
    """ find and return custom transformation by name
        it needs to be set to as actual transform because i do not
        know another way to access the list of non selected transforms """
    bpy.ops.transform.select_orientation(orientation=ct_name)
    return get_selected_custom_transform_by_name(context, ct_name)


def get_selected_custom_transform_by_name(context, ct_name):
    """ find and return an selected custom transformation by name """
    orientation_slot = [
        slot for slot in
        context.scene.transform_orientation_slots
        if slot.custom_orientation
        and ct_name == slot.custom_orientation.name
    ]
    return orientation_slot[0].custom_orientation


def get_first_custom_transform_containing_name(context, ct_name_part):
    """ find and return first custom transformation containing name_part """
    orientation_slot = [
        slot for slot in
        context.scene.transform_orientation_slots
        if slot.custom_orientation
        and ct_name_part in slot.custom_orientation.name
    ]
    return orientation_slot[0].custom_orientation


def rotate_view2custom_transform_z(context, custom_transform_matrix):
    """ rotate view to a custom transformation """
    angle = 0.78539816339745  # 45 degree
    offset_orient_mat = mesh.offfset_orient_for_z(
        custom_transform_matrix.copy(), angle)
    view_rot = offset_orient_mat.to_quaternion()

    r3d = context.area.spaces.active.region_3d
    r3d.view_rotation = view_rot


def rotate_view2custom_transform(context, custom_transform_matrix):
    """ rotate view to a custom transformation """
    view_rot = custom_transform_matrix.to_quaternion()
    r3d = context.area.spaces.active.region_3d
    r3d.view_rotation = view_rot


def rotate_view2custom_transform_z_ole(context, custom_transform):
    """ rotate view to a custom transformation """
    angle = 0.78539816339745  # 45 degree
    offset_orient_mat = mesh.offfset_orient_for_z(
        custom_transform.matrix.copy(), angle)
    view_rot = offset_orient_mat.to_quaternion()

    r3d = context.area.spaces.active.region_3d
    r3d.view_rotation = view_rot


def rotate_view2custom_transform_ole(context, custom_transform):
    """ rotate view to a custom transformation """
    view_rot = custom_transform.matrix.to_quaternion()
    r3d = context.area.spaces.active.region_3d
    r3d.view_rotation = view_rot


def rotate_view2obj(context, obj):
    """ rotate view to a custom transformation """
    view_rot = obj.matrix_local.to_quaternion()
    r3d = context.area.spaces.active.region_3d
    r3d.view_rotation = view_rot


def rotate_view2ct(context, custom_transform):
    """ rotate view to a custom transformation """
    view_rot = custom_transform.matrix_local.to_quaternion()
    r3d = context.area.spaces.active.region_3d
    r3d.view_rotation = view_rot


def center_view2location(context, location):
    """ center view to a custom transformation """
    old_cursor_loc = bpy.context.scene.cursor.location.copy()
    context.scene.cursor.location = location.copy()
    bpy.ops.view3d.view_center_cursor()
    bpy.context.scene.cursor.location = old_cursor_loc
